'use strict';

const keyValue = "45355e9f-51f6-4ac5-8c33-3deb219b153d";
const hostValue = "https://cloud.anylogic.com";
const modelName = "Product Delivery";

let cloudClient = CloudClient.create(keyValue, hostValue);

let animation;
let simulation;
let parameterVariation;
let interval;
let progressInterval;

function runAnimation() {
    console.log(modelName);
    cloudClient.getLatestModelVersion(modelName)
        .then(version => cloudClient.createInputsFromExperiment(version, "Experiment"))
        .then(inputs => cloudClient.startAnimation(inputs, "animation-container"))
        .then(a => {
            animation = a;
            document.getElementById("animationControls").style.display = "";
            interval = setInterval(() => {
                animation.getState().then(state => {
                    document.getElementById("animationState").textContent = state;
                });
            }, 2000);
            return animation.waitForCompletion();
        })
        .then(() => {
            document.getElementById("animationControls").style.display = "none";
            // stop polling: runningAnimation.getState();​
            clearInterval(interval);
        })
        .catch(error => console.error(error));
}

function runSimulation() {
    cloudClient.getLatestModelVersion(modelName)
        .then( version => {
            let inputs = cloudClient.createDefaultInputs( version );
            inputs.setInput( "K", 0.4 );
            inputs.setInput( "{stop_mode}", "STOP_MODE_AT_TIME" );
            simulation = cloudClient.createSimulation(inputs);
            return simulation.getOutputsAndRunIfAbsent([
                "Output"
            ]);
        })
        .then( outputs => {
            console.log(outputs.names());
            console.log(outputs.value(outputs.findNameIncluding("plot")));
            console.log(outputs.getRawOutputs());
            document.getElementById( "text-output" ).innerText = "Output = " + outputs.value("Output");
        })
        .catch( error => console.error(error) );
}

function runParameterVariation() {
    cloudClient.getLatestModelVersion( modelName )
        .then( version => {
            let inputs = cloudClient.createDefaultInputs( version );
            inputs.setRangeInput( "Contact Rate", 21, 41, 20 );
            inputs.setRangeInput( "Adoption Fraction", 0.02, 0.04, 0.01 );
            inputs.setRangeInput( "Ad Effectiveness", 0.015, 0.065, 0.01 );
            console.log(inputs.getInput("Contact Rate"));
            parameterVariation = cloudClient.createParameterVariation(inputs);
            startProgressPolling();
            return parameterVariation.run();
        })
        .then( parameterVariation => parameterVariation.waitForCompletion() )
        .then( parameterVariation => parameterVariation.getOutputs([
            "AdoptionPlot|Potential Adopters",
            "AdoptionPercent"
        ]) )
        .then( outputs => {
            console.log(outputs.getInputNames());
            console.log(outputs.getOutputNames());
            console.log(outputs.getValuesOfInput("Ad Effectiveness"));
            console.log(outputs.getValuesOfOutput("AdoptionPercent"));
            console.log(outputs.getRawData());
            document.getElementById( "text-output" ).innerText = "AdoptionPercent = " + outputs.getValuesOfOutput("AdoptionPercent");
            console.log( "AdoptionPercent = ", outputs.getValuesOfOutput("AdoptionPercent") );
        })
        .catch( error => console.error(error) )
        .finally(() => {
            endProgressPolling();
        });
}

function startProgressPolling() {
    progressInterval = setInterval(() => {
        parameterVariation.getProgress()
            .then(progress => console.log(progress))
    }, 2000);
}

function endProgressPolling() {
    clearInterval(progressInterval);
}

function stopSimulation() {
    simulation.stop();
}
function stopParameterVariation() {
    parameterVariation.stop();
}

function stopAnimation() {
    animation.stop();
}

function pause() {
    animation.pause();
}

function resume() {
    animation.resume();
}

function increaseSpeed() {
    animation.setSpeed(10);
}

function navigateToNotification() {
    animation.navigateTo('notificationArea');
}

function goToAgent() {
    animation.setPresentable('experiment.root.presentableAgent');
}

function setParameters() {
    animation.setValue('experiment.root.parameter1', 42);
    animation.setValue('experiment.root.parameter2', null);
    animation.setValue('experiment.root.parameter3', ['123', '234']);
    animation.setValue('experiment.root.parameter4', {name: "Вася", weight: 111.11});
}

function getParameters() {
    animation.getValue('experiment.root.parameter1')
        .then(value => console.log('parameter1: ', value));

    animation.getValue('experiment.root.parameter2')
        .then(value => console.log('parameter2: ', value));

    animation.getValue('experiment.root.parameter3')
        .then(value => console.log('parameter3: ', value));

    animation.getValue('experiment.root.parameter4')
        .then(value => console.log('parameter4: ', value));

    animation.getValue('experiment.root._plot1_expression0_dataSet_xjal')
        .then(value => console.log('_plot1_expression0_dataSet_xjal: ', value));
}

function callFunction() {
    animation.callFunction('experiment.root.function', [2.42, [10, 30]])
        .then(result => console.log(result));
}
